##############################################################################
##
## Load-Zen-Datatools.ps1 [install|remove}
##
## by Actian Corporation (http://www.actian.com/zen)
##
##############################################################################

    <#
    .NOTES
        Copyright (c) Actian Corporation.  All rights reserved.

        Use of this script is subject to the terms of the Actian Zen license
        agreement under which you licensed the Zen product.  If you did not
        accept the terms of the license agreement, you are not authorized to
        use this script. For the terms of the license, please see the license
        agreement between you and Actian Corporation.
        THE SCRIPT IS PROVIDED "AS IS", WITH NO WARRANTIES.

    .SYNOPSIS
        Integrates the Data Tools for the PSQL ADO.NET Provider v4.4 into
        Visual Studio 2017 or later. Also removes the integration from Visual
        Studio 2017 or later.

    .DESCRIPTION
        Locates the latest installed version of Visual Studio (after VS 2017)
        using the vswhere.exe utility then either installs or removes the
        appropriate Visual Studio Extension package for the Data Tools for PSQL
        ADO.NET Provider v4.4. If Visual Studio is configured for Entity Framework
        development, also copies or deletes the Entity Framework Model First
        templates for the PSQL ADO.NET Provider v4.4 to/from the appropriate
        Visual Studio extensions folder. 

    .PARAMETER install
        Uses the VSIXInstaller.exe utility to install the appropriate PSQL ADO.NET
        Provider Data Tools VSIX package. Also copies the Entity Framework Model
        First templates to the appropriate Visual Studio 2017 or later installation
        location if Visual Studio is configured for Entity Framework development.
        A log file for the VSIXInstaller utility output is created in the %TEMP%
        folder with the name "zen.datatools.vsix.install.log".

    .PARAMETER remove
        Uses the VSIXInstaller.exe utility to uninstall the PSQL ADO.NET Provider
        Data Tools VSIX package. Also deletes the Entity Framework Model First
        templates from the Visual Studio 2017 or later installation location if
        Visual Studio is configured for Entity Framework development. A log file
        for the VSIXInstaller utility output is created in the %TEMP% folder with
        the name "zen.datatools.vsix.uninstall.log".

    .EXAMPLE
        PS C:\> .Load-Zen-Datatools.ps1 install
            Installs the VSIX package and copies the Entity Framework Model First
            templates to Visual Studio 2017 or later.

    .EXAMPLE
        PS C:\> .\Load-Zen-Datatools.ps1 remove
            Removes the installed VSIX package and deletes the Entity Framework Model
            First templates from Visual Studio 2017 or later.

    .INPUTS
        None. You cannot pipe objects to this cmdlet.

    .OUTPUTS
        0 : The script completed with no errors (may contain warnings).
        -1: The script returned one or more errors.

    #>

#Requires -version 3.0

param (
    [Parameter(
        Mandatory=$true,
        Position = 0)]
    [ValidateSet('install','remove')]
    [String]
    $InstallMode
)

Set-StrictMode -Version Latest

Import-LocalizedData msg -FileName "Load-Zen-DataTools.psd1" -ErrorAction Ignore

## Check for administrator rights without using the -RunAsAdministrator parameter for the #Requires
## directive (which requires PowerShell 4 or later).
$CurrentUser = New-Object -TypeName Security.Principal.WindowsPrincipal -ArgumentList $([Security.Principal.WindowsIdentity]::GetCurrent())
if ( ! $CurrentUser.IsInRole([Security.Principal.WindowsBuiltinRole]::Administrator)) {Throw $msg.errMustBeAdmin}

## Check PowerShell process is 64-bit on 64-bit OS.
$is32on64=$env:PROCESSOR_ARCHITEW6432 # Only defined if running 32-bit PS on 64-bit Windows.
if ($is32on64) { Throw $msg.errMustBePS64onWin64 }

## Global variable - exists as long as PoSh session is running.
$global:ZenInfo=""

## Script-level variables definitions.
$scriptName = & {$myInvocation.ScriptName}
$scriptPath = & {Split-Path $myInvocation.ScriptName}
$registryPath = "HKLM:\SOFTWARE\Actian\Zen"
$supportedZenMajorVersionLevel = "14"
$vsixPkgName = Join-Path $scriptPath 'Zen.VisualStudio.DataTools_2017_2019.vsix'
$vsixId='PSQLDataTools.d115023a-8951-4aa5-ae36-94d7b1e6f6ad'
$modelFirstIncNames='GeneratePSQL.Utility*.ttinclude' #ModelFirst Include Template Names
$modelFirstT4Names='SSDLtoPSQL*.tt' # ModelFirst T4 Template Names
$modelFirstIncludes = join-path $scriptPath "Templates\ModelFirstTemplates\$modelFirstIncNames"
$modelFirstT4 = join-path $scriptPath "Templates\ModelFirstTemplates\$modelFirstT4Names"
$vswhere=join-path $scriptPath 'util\vswhere.exe'
$relPathVSIXInstaller='Common7\IDE\VSIXInstaller.exe'
$relPathEntityFrameworkTools='Common7\IDE\Extensions\Microsoft\Entity Framework Tools'
$relPathEntityFrameworkIncludes=join-path $relPathEntityFrameworkTools 'Templates\Includes'
$relPathEntityFrameworkT4=join-path $relPathEntityFrameworkTools 'DBGen'

function Main {
    ##################################################################################################
    ## Invoked as entry-point function at end of script and defines the order the other defined
    ## functions are called.
    ##################################################################################################
    if ($InstallMode -eq "install") {
        FindProductType
        CheckZenVersion
    }
    GetVSInfo
    RunVsixInstaller
    CopyOrRemoveTemplates
}

function FindProductType {
    # NOTE: This function is an exact duplicate of the same function in the winuwp Zen Edit removal
    # script 'Uninstall-Zen.ps1' (starting below this line).
    ##########################
    # Sets the installed product type based on one the 'HKLM:\SOFTWARE\Actian\Zen\InstallInfo'
    # sub-keys for Client, Server, Workgroup.
    #
    # Variables set:
    #   [string] $script:InstalledProductType = client|server|reporting
    #   [object] $global:ZenInfo = object methods match InstallInfo registry values.
    #                   $ZenInfo.InstallData
    #                   $ZenInfo.InstallDir64
    #                   $ZenInfo.InstallDir
    #                   $ZenInfo.IsBtrieve
    #                   $ZenInfo.IsReport
    #                   $ZenInfo.IsVx
    #                   $ZenInfo.ProductVersion
    #                   $ZenInfo.VersionLevel
    #                   $ZenInfo.InstallData
    #
    # Note: Must be called before logging is started.
    ##########################
    $script:InstalledProductType=""
    if (test-path $registryPath"\InstallInfo\") {
        Get-ChildItem $registryPath"\InstallInfo\" | ForEach-Object -Process {
        Write-Host "Name: "  $_
            switch ($_.PSChildName) {
                'Client' {
                    $global:ZenInfo=Get-ItemProperty $registryPath"\InstallInfo\$_"
                    $script:InstalledProductType="client"
                    break}
                'Server' {
                    $global:ZenInfo=Get-ItemProperty $registryPath"\InstallInfo\$_"
                    if ($ZenInfo.IsVx -eq 0) {
                        $script:InstalledProductType="server"
                    } else {
                        $script:InstalledProductType="vx"
                    }
                    break}
                'Workgroup' {
                    $global:ZenInfo=Get-ItemProperty $registryPath"\InstallInfo\$_"
                    if ($ZenInfo.IsReport -eq 0) {
                        $script:InstalledProductType="workgroup"
                    } else {
                        $script:InstalledProductType="reporting"
                    }
                    break}
            }
        }
    }
}

function CheckZenVersion {
    ##################################################################################################
    ## Verifies the major version specified in $ZenInfo.ProductVersion (global object set by previous
    ## call to FindProductType) starts with the major version number defined for by script variable
    ## $supportedZenMajorVersionLevel. The script is aborted if the detected major version is not
    ## the same as the defined major version.
    ##################################################################################################
    if ($ZenInfo -ne "") {
        $msg.infoZenInfo -f $ZenInfo.ProductVersion| Write-Output
        $currentVersion=$ZenInfo.ProductVersion[0]+$ZenInfo.ProductVersion[1]
        if ("$supportedZenMajorVersionLevel" -gt $currentVersion ) {
           $msg.errWrongZenVer -f $supportedZenMajorVersionLevel | Write-Error;exit -1
        }
    } else {
        $msg.errNoZenFound | Write-Error;exit -1
    }
}

function GetVSInfo {
    ##################################################################################################
    ## Creates the script level PoSh object "vsInfo" with the member properties specified below. If
    ## Visual Studio 2017 or later is not detected the script displays an error message and aborts.
    ## If the retrieved path to the VSIXInstaller utility is invalid the script also displays an error
    ## message and aborts.
    ##
    ## object member properties:
    ##    vsInfo.installDir    : value of property 'installationPath' returned by vswhere utility.
    ##    vsInfo.model1_inc    : path to Entity Framework Tools\Templates\Includes folder if it exists,
    ##                           empty string if it does not exist.
    ##    vsInfo.model1_t4     : path to Entity Framework Tools\DBGen folder if it exists, empty string
    ##                           if it does not exist.
    ##    vsInfo.version       : value of property 'installationVersion' returned by vswhere utility.
    ##    vsInfo.vsixInstaller : path to the VSIXInstaller utility (must exist).
    ##################################################################################################
    $vsInstallDirFound=$false
     
    ## Get installed version and location of VS. Vswhere requires the '-legacy' option for VS
    ## versions prior to VS 2017 (v15.0.#####.##).
    $vsVersion=& $vswhere -nologo -latest -products * -property installationVersion
    if ($vsVersion) {
        ## VS 2017 or later.
        $vsInstallDir=& $vswhere -nologo -latest -products * -property installationPath
        if ($vsInstallDir -and (Test-Path -Path $vsInstallDir)) {$vsInstallDirFound=$True}
    }

    ## Abort script if $vsVersion is not "15.0" or later (Visual Studio 2017 or later) or if the 
    ## installation path does not exist.
    if ( (!$vsVersion) -or ($vsVersion -lt "15.0") -or (!$vsInstallDirFound) ) {$msg.errNoVSFound -f "2017"| Write-Error;exit -1}

    ## Verify the location of the VSIXInstaller utility required to install/remove the VSIX package.
    $vsixInstaller=Join-Path $vsInstallDir $relPathVSIXInstaller
    if (! (test-path $vsixInstaller)) {$msg.errNoVSIXInstaller | Write-Error;exit -1}
     
    ## Verify the location of the Entity Framework Tools "Templates\Includes" and "DBGen" folders
    ## (sets corresponding vsInfo properties to empty string if the folder doesn't exist).
    $entityFrameworkIncludesPath=Join-Path $vsInstallDir $relPathEntityFrameworkIncludes
    if (!(test-path $entityFrameworkIncludesPath)) {$entityFrameworkIncludesPath=""}
    $entityFrameworkDBGenPath=Join-Path $vsInstallDir $relPathEntityFrameworkT4
    if (!(test-path $entityFrameworkDBGenPath)) {$entityFrameworkDBGenPath=""}

    ## Create PoSh object vsInfo.
    $script:vsInfo = @{
        'installDir'=$vsInstallDir;
        'model1_inc'=$entityFrameworkIncludesPath;
        'model1_t4'=$entityFrameworkDBGenPath;
        'version'=$vsVersion;
        'vsixInstaller'=$vsixInstaller;
    }
     
    ## Output VS version and location info.
    $msg.infoVisualStudioInfo -f $vsInfo.version,$vsInfo.installDir | Write-Output
}

function RunVsixInstaller {
    ##################################################################################################
    ## Runs VSIXInstaller utility to install or remove the appropriate VSIX package.  If the
    ## VSIXInstaller.exe process cannot be started an error message is displayed and the script is
    ## aborted, otherwise an informational message instructing the user to check the VSIXInstaller log
    ## file is displayed.
    ##################################################################################################

    ## Set the VSIX package name and install/uninstall options based on Visual Studio version.
    $vsixInstallArgs="/quiet","/logFile:Zen.DataTools.VSIX.install.log","$vsixPkgName"
    $vsixInstallMsg=$msg.infoInstalledVsixPkgLatestWithLog -f [string]"Zen.DataTools.VSIX.install.log"
    $vsixInstallErr=$msg.errVsixInstallWithLog -f [string]" (Zen.DataTools.VSIX.install.log)"
    $vsixRemovalArgs="/quiet","/logFile:Zen.DataTools.VSIX.uninstall.log","/uninstall:$vsixId"
    $vsixRemovalMsg=$msg.infoRemovedVsixPkgLatestWithLog -f [string]"Zen.DataTools.VSIX.uninstall.log"
    $vsixRemovalErr=$msg.errVsixUninstallWithLog -f [string]" (Zen.DataTools.VSIX.uninstall.log)"


    ## Install or remove the VSIX package based on $InstallMode.
    if ($InstallMode -eq "install") {
        Try {
           $msg.infoInstallingVsixPkg | Write-Output
           Start-Process -FilePath $vsInfo.vsixInstaller -ArgumentList $vsixInstallArgs -NoNewWindow -Wait
           $vsixInstallMsg | Write-Output
        }
        Catch {$vsixInstallErr | Write-Error;exit -1}
    } else {
        Try {
            $msg.infoRemovingVsixPkg | Write-Output
            Start-Process -FilePath $vsInfo.vsixInstaller -ArgumentList $vsixRemovalArgs -NoNewWindow -Wait
            $vsixRemovalMsg | Write-Output
        }
        Catch {$vsixRemovalErr | Write-Error;exit -1}
    } 
}

function CopyOrRemoveTemplates {
    ##################################################################################################
    ## Copies or removes the Entity Framework Model First templates to/from the appropriate Visual
    ## Studio folder.  If the path to the Entity Framework Tools folder cannot be located this step
    ## is skipped.
    ##################################################################################################

    if ($InstallMode -eq "install") {  ## Copy templates.
        Try {
            ## Verify Entity Framework Tools folder and copy templates.
            $entityFrameworkTools=join-path $vsInfo.installDir $relPathEntityFrameworkTools
            if (Test-Path $entityFrameworkTools) {

                ## Model First include templates.
                Copy-Item -Path $modelFirstIncludes -Destination $vsInfo.model1_inc -Force
                if ($?) {$msg.infoAddedModelFirstIncludeTemplates | Write-Output}

                ## Model First T4 templates.
                Copy-Item -Path $modelFirstT4 -Destination $vsInfo.model1_t4 -Force
                if ($?) {$msg.infoAddedModelFirstT4Templates | Write-Output}

            }
        }
        Catch {$msg.warnNoTemplates -f [string]"$relPathEntityFrameworkTools" | Write-Warning}
    } else {  ## Remove templates.
        Try {
            ## Verify Model First include templates exist and remove them.
            $removeIncludes=join-path $vsInfo.model1_inc $modelFirstIncNames
            if (Test-Path $removeIncludes) {
                remove-item -Path $removeIncludes -Force
                if ($?) {$msg.infoRemovedModelFirstIncludeTemplates | Write-Output}
            }

            ## Verify Model First T4 templates exist and remove them.
            $removeT4=join-path $vsInfo.model1_t4 $modelFirstT4Names
            if (Test-Path $removeT4) {
                remove-item -Path $removeT4 -Force
                if ($?) {$msg.infoRemovedModelFirstT4Templates | Write-Output}
            }
        }
        Catch {$msg.warnNoTemplates -f [string]"$relPathEntityFrameworkTools" | Write-Warning}
    }
}

## Invoke Main function.
"" | Write-Output
$msg.infoRunning -f $MyInvocation.InvocationName, $InstallMode | Write-Output
. Main
$msg.infoEnd -f $MyInvocation.InvocationName, $InstallMode | Write-Output
"" | Write-Output
exit 0
# SIG # Begin signature block
# MIIZeQYJKoZIhvcNAQcCoIIZajCCGWYCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUlw5Hk3RuBVN2sOEvk+cgQFtJ
# QragghQ5MIIE0DCCA7igAwIBAgIBBzANBgkqhkiG9w0BAQsFADCBgzELMAkGA1UE
# BhMCVVMxEDAOBgNVBAgTB0FyaXpvbmExEzARBgNVBAcTClNjb3R0c2RhbGUxGjAY
# BgNVBAoTEUdvRGFkZHkuY29tLCBJbmMuMTEwLwYDVQQDEyhHbyBEYWRkeSBSb290
# IENlcnRpZmljYXRlIEF1dGhvcml0eSAtIEcyMB4XDTExMDUwMzA3MDAwMFoXDTMx
# MDUwMzA3MDAwMFowgbQxCzAJBgNVBAYTAlVTMRAwDgYDVQQIEwdBcml6b25hMRMw
# EQYDVQQHEwpTY290dHNkYWxlMRowGAYDVQQKExFHb0RhZGR5LmNvbSwgSW5jLjEt
# MCsGA1UECxMkaHR0cDovL2NlcnRzLmdvZGFkZHkuY29tL3JlcG9zaXRvcnkvMTMw
# MQYDVQQDEypHbyBEYWRkeSBTZWN1cmUgQ2VydGlmaWNhdGUgQXV0aG9yaXR5IC0g
# RzIwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQC54MsQ1K92vdSTYusw
# ZLiBCGzDBNliF44v/z5lz4/OYuY8UhzaFkVLVat4a2ODYpDOD2lsmcgaFItMzEUz
# 6ojcnqOvK/6AYZ15V8TPLvQ/MDxdR/yaFrzDN5ZBUY4RS1T4KL7QjL7wMDge87Am
# +GZHY23ecSZHjzhHU9FGHbTj3ADqRay9vHHZqm8A29vNMDp5T19MR/gd71vCxJ1g
# O7GyQ5HYpDNO6rPWJ0+tJYqlxvTV0KaudAVkV4i1RFXULSo6Pvi4vekyCgKUZMQW
# OlDxSq7neTOvDCAHf+jfBDnCaQJsY1L6d8EbyHSHyLmTGFBUNUtpTrw700kuH9zB
# 0lL7AgMBAAGjggEaMIIBFjAPBgNVHRMBAf8EBTADAQH/MA4GA1UdDwEB/wQEAwIB
# BjAdBgNVHQ4EFgQUQMK9J47MNIMwojPX+2yz8LQsgM4wHwYDVR0jBBgwFoAUOpqF
# BxBnKLbv9r0FQW4gwZTaD94wNAYIKwYBBQUHAQEEKDAmMCQGCCsGAQUFBzABhhho
# dHRwOi8vb2NzcC5nb2RhZGR5LmNvbS8wNQYDVR0fBC4wLDAqoCigJoYkaHR0cDov
# L2NybC5nb2RhZGR5LmNvbS9nZHJvb3QtZzIuY3JsMEYGA1UdIAQ/MD0wOwYEVR0g
# ADAzMDEGCCsGAQUFBwIBFiVodHRwczovL2NlcnRzLmdvZGFkZHkuY29tL3JlcG9z
# aXRvcnkvMA0GCSqGSIb3DQEBCwUAA4IBAQAIfmyTEMg4uJapkEv/oV9PBO9sPpyI
# BslQj6Zz91cxG7685C/b+LrTW+C05+Z5Yg4MotdqY3MxtfWoSKQ7CC2iXZDXtHwl
# TxFWMMS2RJ17LJ3lXubvDGGqv+QqG+6EnriDfcFDzkSnE3ANkR/0yBOtg2DZ2HKo
# cyQetawiDsoXiWJYRBuriSUBAA/NxBti21G00w9RKpv0vHP8ds42pM3Z2Czqrpv1
# KrKQ0U11GIo/ikGQI31bS/6kA1ibRrLDYGCD+H1QQc7CoZDDu+8CL9IVVO5EFdkK
# rqeKM+2xLXY2JtwE65/3YR8V3Idv7kaWKK2hJn0KCacuBKONvPi8BDABMIIE/jCC
# A+agAwIBAgIQDUJK4L46iP9gQCHOFADw3TANBgkqhkiG9w0BAQsFADByMQswCQYD
# VQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGln
# aWNlcnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFzc3VyZWQgSUQgVGlt
# ZXN0YW1waW5nIENBMB4XDTIxMDEwMTAwMDAwMFoXDTMxMDEwNjAwMDAwMFowSDEL
# MAkGA1UEBhMCVVMxFzAVBgNVBAoTDkRpZ2lDZXJ0LCBJbmMuMSAwHgYDVQQDExdE
# aWdpQ2VydCBUaW1lc3RhbXAgMjAyMTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCC
# AQoCggEBAMLmYYRnxYr1DQikRcpja1HXOhFCvQp1dU2UtAxQtSYQ/h3Ib5FrDJbn
# GlxI70Tlv5thzRWRYlq4/2cLnGP9NmqB+in43Stwhd4CGPN4bbx9+cdtCT2+anaH
# 6Yq9+IRdHnbJ5MZ2djpT0dHTWjaPxqPhLxs6t2HWc+xObTOKfF1FLUuxUOZBOjdW
# htyTI433UCXoZObd048vV7WHIOsOjizVI9r0TXhG4wODMSlKXAwxikqMiMX3MFr5
# FK8VX2xDSQn9JiNT9o1j6BqrW7EdMMKbaYK02/xWVLwfoYervnpbCiAvSwnJlaeN
# svrWY4tOpXIc7p96AXP4Gdb+DUmEvQECAwEAAaOCAbgwggG0MA4GA1UdDwEB/wQE
# AwIHgDAMBgNVHRMBAf8EAjAAMBYGA1UdJQEB/wQMMAoGCCsGAQUFBwMIMEEGA1Ud
# IAQ6MDgwNgYJYIZIAYb9bAcBMCkwJwYIKwYBBQUHAgEWG2h0dHA6Ly93d3cuZGln
# aWNlcnQuY29tL0NQUzAfBgNVHSMEGDAWgBT0tuEgHf4prtLkYaWyoiWyyBc1bjAd
# BgNVHQ4EFgQUNkSGjqS6sGa+vCgtHUQ23eNqerwwcQYDVR0fBGowaDAyoDCgLoYs
# aHR0cDovL2NybDMuZGlnaWNlcnQuY29tL3NoYTItYXNzdXJlZC10cy5jcmwwMqAw
# oC6GLGh0dHA6Ly9jcmw0LmRpZ2ljZXJ0LmNvbS9zaGEyLWFzc3VyZWQtdHMuY3Js
# MIGFBggrBgEFBQcBAQR5MHcwJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2lj
# ZXJ0LmNvbTBPBggrBgEFBQcwAoZDaHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29t
# L0RpZ2lDZXJ0U0hBMkFzc3VyZWRJRFRpbWVzdGFtcGluZ0NBLmNydDANBgkqhkiG
# 9w0BAQsFAAOCAQEASBzctemaI7znGucgDo5nRv1CclF0CiNHo6uS0iXEcFm+FKDl
# J4GlTRQVGQd58NEEw4bZO73+RAJmTe1ppA/2uHDPYuj1UUp4eTZ6J7fz51Kfk6ft
# Q55757TdQSKJ+4eiRgNO/PT+t2R3Y18jUmmDgvoaU+2QzI2hF3MN9PNlOXBL85zW
# envaDLw9MtAby/Vh/HUIAHa8gQ74wOFcz8QRcucbZEnYIpp1FUL1LTI4gdr0YKK6
# tFL7XOBhJCVPst/JKahzQ1HavWPWH1ub9y4bTxMd90oNcX6Xt/Q/hOvB46NJofrO
# p79Wz7pZdmGJX36ntI5nePk2mOHLKNpbh6aKLzCCBSowggQSoAMCAQICCCb6uKSN
# FCdLMA0GCSqGSIb3DQEBCwUAMIG0MQswCQYDVQQGEwJVUzEQMA4GA1UECBMHQXJp
# em9uYTETMBEGA1UEBxMKU2NvdHRzZGFsZTEaMBgGA1UEChMRR29EYWRkeS5jb20s
# IEluYy4xLTArBgNVBAsTJGh0dHA6Ly9jZXJ0cy5nb2RhZGR5LmNvbS9yZXBvc2l0
# b3J5LzEzMDEGA1UEAxMqR28gRGFkZHkgU2VjdXJlIENlcnRpZmljYXRlIEF1dGhv
# cml0eSAtIEcyMB4XDTIwMDQxMzIwMDc0MVoXDTIzMDQxMzIwMDc0MVowbDELMAkG
# A1UEBhMCVVMxDjAMBgNVBAgTBVRleGFzMRMwEQYDVQQHEwpSb3VuZCBSb2NrMRsw
# GQYDVQQKExJBY3RpYW4gQ29ycG9yYXRpb24xGzAZBgNVBAMTEkFjdGlhbiBDb3Jw
# b3JhdGlvbjCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBALyocy/QINa6
# HjQBRinT3Gq+AgGqcJwsbsA/K5ZELJgmTuO4XIbSm/pK2DKhj0/KPu62GR2CzOaM
# Wc2aHCt1k6BA2f4mVdQYhh1VF2Awc0WFhLKdISJQeKY2m+bYxxBWa0BmK6/O6R11
# nn0w6N2fiMs/CSoA8Xkug8Ee+oFMB8Pgct4TFU/xFu7TP7kCIXpWNc2blMoAWLFq
# TWv3DfCbiZmlgHnI8YeD5PPneOk7P9h/A1+sdfU6PzE6z/R1+wi6OI2U/TzkDYVx
# F7+LMUZaxpDP+KIWUDwwKLLYNWxIuM4MrZ/skEewc/DFzgYRbrGxWTsytLqrOnuB
# AhMHMYAEFNkCAwEAAaOCAYUwggGBMAwGA1UdEwEB/wQCMAAwEwYDVR0lBAwwCgYI
# KwYBBQUHAwMwDgYDVR0PAQH/BAQDAgeAMDUGA1UdHwQuMCwwKqAooCaGJGh0dHA6
# Ly9jcmwuZ29kYWRkeS5jb20vZ2RpZzJzNS01LmNybDBdBgNVHSAEVjBUMEgGC2CG
# SAGG/W0BBxcCMDkwNwYIKwYBBQUHAgEWK2h0dHA6Ly9jZXJ0aWZpY2F0ZXMuZ29k
# YWRkeS5jb20vcmVwb3NpdG9yeS8wCAYGZ4EMAQQBMHYGCCsGAQUFBwEBBGowaDAk
# BggrBgEFBQcwAYYYaHR0cDovL29jc3AuZ29kYWRkeS5jb20vMEAGCCsGAQUFBzAC
# hjRodHRwOi8vY2VydGlmaWNhdGVzLmdvZGFkZHkuY29tL3JlcG9zaXRvcnkvZ2Rp
# ZzIuY3J0MB8GA1UdIwQYMBaAFEDCvSeOzDSDMKIz1/tss/C0LIDOMB0GA1UdDgQW
# BBQwB/xOgqq0irDx1sDDAdjdl8kxIjANBgkqhkiG9w0BAQsFAAOCAQEAXPCXCcUa
# Mshqi7pczfhn5VbNDv4wEvcdsLFdCIgFhSQehsjdEelL9hrwrlfQZNag6UK5eavs
# zK81RzUKmNS49S48dQDhpp+3B9jDmyIIed8wop6T+B9z50vmJ6+nvlZd7ylvYef3
# v47q2m1PiUPBu4fXNEL8LjD66s7Iw5yjy1g0e5k0VXhSc4i6BacdGZ4V2z0a2Q1g
# Jky/PGnKKyqI0iPOzU6X1E9iTEKKN6Gw7lrOb1uCQlEa9TTJDMVi2I5EkEjHHE24
# mNdqqh05cF/8AmwWXBG5U3iI5414F21ibgUElxguFZdQJTOID6TxHYyfcnurBXcH
# EJSK9Dp8MWB77zCCBTEwggQZoAMCAQICEAqhJdbWMht+QeQF2jaXwhUwDQYJKoZI
# hvcNAQELBQAwZTELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZ
# MBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEkMCIGA1UEAxMbRGlnaUNlcnQgQXNz
# dXJlZCBJRCBSb290IENBMB4XDTE2MDEwNzEyMDAwMFoXDTMxMDEwNzEyMDAwMFow
# cjELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQ
# d3d3LmRpZ2ljZXJ0LmNvbTExMC8GA1UEAxMoRGlnaUNlcnQgU0hBMiBBc3N1cmVk
# IElEIFRpbWVzdGFtcGluZyBDQTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoC
# ggEBAL3QMu5LzY9/3am6gpnFOVQoV7YjSsQOB0UzURB90Pl9TWh+57ag9I2ziOSX
# v2MhkJi/E7xX08PhfgjWahQAOPcuHjvuzKb2Mln+X2U/4Jvr40ZHBhpVfgsnfsCi
# 9aDg3iI/Dv9+lfvzo7oiPhisEeTwmQNtO4V8CdPuXciaC1TjqAlxa+DPIhAPdc9x
# ck4Krd9AOly3UeGheRTGTSQjMF287DxgaqwvB8z98OpH2YhQXv1mblZhJymJhFHm
# gudGUP2UKiyn5HU+upgPhH+fMRTWrdXyZMt7HgXQhBlyF/EXBu89zdZN7wZC/aJT
# Kk+FHcQdPK/P2qwQ9d2srOlW/5MCAwEAAaOCAc4wggHKMB0GA1UdDgQWBBT0tuEg
# Hf4prtLkYaWyoiWyyBc1bjAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYunpyGd823I
# DzASBgNVHRMBAf8ECDAGAQH/AgEAMA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAK
# BggrBgEFBQcDCDB5BggrBgEFBQcBAQRtMGswJAYIKwYBBQUHMAGGGGh0dHA6Ly9v
# Y3NwLmRpZ2ljZXJ0LmNvbTBDBggrBgEFBQcwAoY3aHR0cDovL2NhY2VydHMuZGln
# aWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNydDCBgQYDVR0fBHow
# eDA6oDigNoY0aHR0cDovL2NybDQuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJl
# ZElEUm9vdENBLmNybDA6oDigNoY0aHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0Rp
# Z2lDZXJ0QXNzdXJlZElEUm9vdENBLmNybDBQBgNVHSAESTBHMDgGCmCGSAGG/WwA
# AgQwKjAoBggrBgEFBQcCARYcaHR0cHM6Ly93d3cuZGlnaWNlcnQuY29tL0NQUzAL
# BglghkgBhv1sBwEwDQYJKoZIhvcNAQELBQADggEBAHGVEulRh1Zpze/d2nyqY3qz
# eM8GN0CE70uEv8rPAwL9xafDDiBCLK938ysfDCFaKrcFNB1qrpn4J6JmvwmqYN92
# pDqTD/iy0dh8GWLoXoIlHsS6HHssIeLWWywUNUMEaLLbdQLgcseY1jxk5R9IEBhf
# iThhTWJGJIdjjJFSLK8pieV4H9YLFKWA1xJHcLN11ZOFk362kmf7U2GJqPVrlsD0
# WGkNfMgBsbkodbeZY4UijGHKeZR+WfyMD+NvtQEmtmyl7odRIeRYYJu6DC0rbaLE
# frvEJStHAgh8Sa4TtuF8QkIoxhhWz0E0tmZdtnR79VYzIi8iNrJLokqV2PWmjlIx
# ggSqMIIEpgIBATCBwTCBtDELMAkGA1UEBhMCVVMxEDAOBgNVBAgTB0FyaXpvbmEx
# EzARBgNVBAcTClNjb3R0c2RhbGUxGjAYBgNVBAoTEUdvRGFkZHkuY29tLCBJbmMu
# MS0wKwYDVQQLEyRodHRwOi8vY2VydHMuZ29kYWRkeS5jb20vcmVwb3NpdG9yeS8x
# MzAxBgNVBAMTKkdvIERhZGR5IFNlY3VyZSBDZXJ0aWZpY2F0ZSBBdXRob3JpdHkg
# LSBHMgIIJvq4pI0UJ0swCQYFKw4DAhoFAKCBijAZBgkqhkiG9w0BCQMxDAYKKwYB
# BAGCNwIBBDAcBgorBgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIBFTAjBgkqhkiG9w0B
# CQQxFgQUUIM5kXI5IWv3rrWuHVomr++VvsEwKgYKKwYBBAGCNwIBDDEcMBqhGIAW
# aHR0cDpcXHd3dy5hY3RpYW4uY29tIDANBgkqhkiG9w0BAQEFAASCAQAxgcWY9cGh
# Y2lwfC90Qj++b1W2sK4YYUrL7XP25LWIWvptAwodrWancuFzgq/ovinql3IxORff
# 3dGi3rSSFRc9I8Vj67tmyyFQbzjDT/ozpfteAG/tX2uQQNtYS8qAPR796kyWYWeD
# 2HuLydUAFSWaOvyoHdYzU1keCTNh0th3hraKG1k8nRCndbtgZN/2L/S6AF+Q+R0v
# C0qo+VUPOE3TwmNmCO0FajHVFhHZ3YJbMGqp0raznHPWHyUBEvi1Xd8A4GSC8BX+
# XH/zNxUVjJT8+NVnRu3ceZhFgxEqFnZxtb8fwjwsGvyUBaRGZrVueWUiWoCRV/DW
# Cnd7UgA3R2gioYICMDCCAiwGCSqGSIb3DQEJBjGCAh0wggIZAgEBMIGGMHIxCzAJ
# BgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5k
# aWdpY2VydC5jb20xMTAvBgNVBAMTKERpZ2lDZXJ0IFNIQTIgQXNzdXJlZCBJRCBU
# aW1lc3RhbXBpbmcgQ0ECEA1CSuC+Ooj/YEAhzhQA8N0wDQYJYIZIAWUDBAIBBQCg
# aTAYBgkqhkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJBTEPFw0yMTEw
# MDcyMjA5MzNaMC8GCSqGSIb3DQEJBDEiBCCFPN3YA+quKDGdtH7wteHw1iGgswbf
# wGJc/m9UPjUOIzANBgkqhkiG9w0BAQEFAASCAQBHflQPsvI41ogGZEYmOzUH9cVa
# lBD/cS8wpWJBWsHXaSqRgHu4EGqbYNrQIIECJ3DI2lgy/yk+PJquLbnTD5JBkgnz
# yreQ0mjaf8PywgIvGBJiGGGds2GPl38i6XNsu+ACeq7Jjok+EcoW4IhwlbpPnqE2
# +J0ipVnb/MX87XVmzqJAySjEiy/ZPP5iYd7tBSbfPsntfzpbWTwtLx2CT4O1/q4o
# hYkE9BXy7P/PjdXa4hSwJxLsRdIBeEfoPPU6rp38a8VnVBwrJm9zgEkrmaZUH35G
# mMwfFpFPNnxwZiW8qAiWz2ThaBX1XjSu7dm3dCdd6pU7dQN/GFd8R37DpGIB
# SIG # End signature block
